/***************************************************************************
 Example Code of a  sumobot with 2 line trackers (one at the front and another at the back) and an ultrasonic front obstacle sensors
 
 This software is free provided that this notice is not removed and proper attribution 
 is accorded to Layad Circuits and its Author(s).
 Layad Circuits invests resources in producing free software. By purchasing Layad Circuits'
 products or utilizing its services, you support the continuing development of free software
 for all.
  
 Author(s): C.D.M. for Layad Circuits Electronics Engineering
 Revision: 1.1.1 - L298N version
 Layad Circuits Electronics Engineering Supplies and Services
 B314 Lopez Bldg., Session Rd. cor. Assumption Rd., Baguio City, Philippines
 www.layadcircuits.com
 general: info@layadcircuits.com
 sales: sales@layadcircuits.com
 +63-916-442-8565
 ***************************************************************************/


// change below to define if line 
// is black on white background or
// white on black background.
// These define the sensor value when
// under black and white surface
// You may also use the DO and ~DO
// pins of the Saleng Tracker to 
// change line configuration

#define BLK HIGH  // line
#define WHT LOW // background
#define DISTANCE_THRESHOLD_BEFORE_RAMMING 50 // distance in cm below which the robot assumes opponent is directly infront of it

// speed setting based on movement
// 255=full speed
#define SPEED_ON_TURNS 80
#define SPEED_FORWARD 255
#define SPEED_BACKWARD 255

#define AIN1 2 
#define BIN1 7
#define AIN2 4
#define BIN2 5
#define PWMA 3
#define PWMB 6

#define US_TRIG 12
#define US_ECHO A3
#define FRONT_LINE_SENSOR 10 // "Right" in linetracing mode
#define REAR_LINE_SENSOR 11  // "Center" in linetracing mode

unsigned long us_dist, ir_dist;

void speedSetting(byte val)
{
  analogWrite(PWMA,val);
  analogWrite(PWMB,val);
}

void forward()
{
  speedSetting(SPEED_FORWARD);
  digitalWrite(AIN1,LOW);
  digitalWrite(AIN2,HIGH);
  digitalWrite(BIN1,LOW);
  digitalWrite(BIN2,HIGH);
}

void backward()
{
  speedSetting(SPEED_BACKWARD);
  digitalWrite(AIN1,HIGH);
  digitalWrite(AIN2,LOW);
  digitalWrite(BIN1,HIGH);
  digitalWrite(BIN2,LOW);
}

void turnleft()
{
  speedSetting(SPEED_ON_TURNS);
  digitalWrite(AIN1,LOW);
  digitalWrite(AIN2,HIGH);
  digitalWrite(BIN1,HIGH);
  digitalWrite(BIN2,LOW);
}

void turnright()
{
  speedSetting(SPEED_ON_TURNS);
  digitalWrite(AIN1,HIGH);
  digitalWrite(AIN2,LOW);
  digitalWrite(BIN1,LOW);
  digitalWrite(BIN2,HIGH);
}

void motorstop()
{
  digitalWrite(AIN1,LOW);
  digitalWrite(AIN2,LOW);
  digitalWrite(BIN1,LOW);
  digitalWrite(BIN2,LOW);
}

void shortbreak()
{
  digitalWrite(AIN1,HIGH);
  digitalWrite(AIN2,HIGH);
  digitalWrite(BIN1,HIGH);
  digitalWrite(BIN2,HIGH);
}




unsigned long us_sensor()
{
  static unsigned long t;
  if(millis() - t < 50) return;
  t = millis();
  
  unsigned long d;
  float f;
  digitalWrite(US_TRIG,LOW);
  delayMicroseconds(20);
  digitalWrite(US_TRIG,HIGH); 
  delayMicroseconds(100);
  digitalWrite(US_TRIG,LOW); 
  d = pulseIn(US_ECHO,HIGH, 50000);
  f = ( (float)d / 29.4 ) / 2;
  d = (unsigned long)f;
  if( d ==0 ) d = 200;
  
  return d;
}

// generate a number 0-1
byte generate_random()
{
  randomSeed(micros()%100);
  return   random(0,1);
}



void setup() {
  pinMode(US_TRIG,OUTPUT);
  pinMode(US_ECHO,INPUT);
  pinMode(FRONT_LINE_SENSOR, INPUT);
  pinMode(REAR_LINE_SENSOR, INPUT);

  pinMode(AIN1,OUTPUT);
  pinMode(AIN2,OUTPUT);
  pinMode(BIN1,OUTPUT);
  pinMode(BIN2,OUTPUT);
  
  // we use a low power to avoid
  // overshooting lines
  // and conserve battery
  // full POWER = 255
  speedSetting(200); 
  delay(1000); // add 1s delay 
  Serial.begin(115200);
  
}


void loop() {

  if(digitalRead(FRONT_LINE_SENSOR) == HIGH) // front sensor was tripped
  {
      // move back
      backward();
      delay(500);//adjust as needed
  }
  else if(digitalRead(REAR_LINE_SENSOR) == HIGH)// rear sensor was tripped
  {
    // move forward
    forward();
    delay(500);//adjust as needed
  }
  else // no line sensors were touched so do scan algorithm
  {
    us_dist = us_sensor();
    
    if(us_dist < DISTANCE_THRESHOLD_BEFORE_RAMMING )
    {
      forward(); 
    }
    else
    {
      turnleft(); // or turnright()
      us_dist = us_sensor();
    }
  }
} 
